﻿using CertificateAutorityServer.Model.Rest;
using CertificateAutorityServer.Service;
using Microsoft.AspNetCore.Mvc;

namespace CertificateAutorityServer.Controllers.Rca
{
    [ApiController]
    [Route("rca")]
    public class RcaController : Controller
    {
        public RcaController(CertificateService certificateService)
        {
            this.certificateService = certificateService;
        }
        private readonly CertificateService certificateService;

        [HttpGet]
        public async Task<IActionResult> GetAsPem()
        {
            var result = await certificateService.GetRcaAsPem();
            return Ok(result);
        }

        [HttpPost("csr")]
        public async Task<IActionResult> ReqCsrSign(SignCsrReq signCsrReq)
        {
            var result = await certificateService.SignCsrByRca(signCsrReq);
            return string.IsNullOrEmpty(result) ? BadRequest() : Ok(result);
        }

        [HttpPost("check")]
        public async Task<IActionResult> CheckCrtByRca(CheckCrtReq checkCrtReq)
        {
            var result = await certificateService.CheckCsrByRca(checkCrtReq.Crt);
            return result ? Ok() : Problem();
        }

        [HttpPost("regenerate")]
        public IActionResult ReCreateRca(bool addTrusted = true)
        {
            certificateService.ReGenerateRca(addTrusted);
            return Ok();
        }
    }
}
