﻿using CAUtilLib;
using CertificateAutorityServer.Model.Rest;
using CertificateAutorityServer.Service;
using Microsoft.AspNetCore.Mvc;

namespace CertificateAutorityServer.Controllers
{
    [ApiController]
    [Route("crt")]
    public class RootController : Controller
    {
        public RootController(CertificateService certificateService) {
            this.certificateService = certificateService;
        }
        private readonly CertificateService certificateService;

        [HttpGet("key")]
        public IActionResult CreatNewKey(int length = 4096) {
            var key = certificateService.GenerateRsaPrivateKeyPemString(length);
            return Ok(key);
        }

        [HttpGet("rca")]
        public async Task<IActionResult> CreatSelfSignedCertificate([FromForm] CreateRootCaReq createRootCaReq)
        {
            var subject = BouncyCastleWrapper.CreateX509Name(
                commonName: createRootCaReq.CommonName,
                organizationName: createRootCaReq.OrganizationName,
                country: createRootCaReq.Country,
                state: createRootCaReq.State);

            var key = BouncyCastleWrapper.LoadPemKeyFromString(createRootCaReq.Key);
            if (key is null)
            {
                return BadRequest("key parse failed");
            }
            //var cert = BouncyCastleWrapper.GenerateCertificate(subject, subject, key, key.Public);
            var cert = BouncyCastleWrapper.GenerateSelfSignedRootCertificate(subject, key);
            if (cert is null)
            {
                return BadRequest("certificate create failed");
            }
            var certStrig = await BouncyCastleWrapper.ToStringAsPem(cert);
            return Ok(certStrig);
        }

        [HttpGet("csr")]
        public IActionResult CreatCsr([FromForm] CreateCsrReq createCsrReq)
        {
            var csr = certificateService.CreateCertificateReq(createCsrReq);
            return Ok(csr);
        }

        [HttpGet("crt")]
        public async Task<IActionResult> CreatCrt([FromForm] SignCsrReq signCsrReq)
        {
            var csr = BouncyCastleWrapper.LoadPemCsrFromString(signCsrReq.Csr);
            var rca = BouncyCastleWrapper.LoadPemCertFromString(signCsrReq.Rca);
            var key = BouncyCastleWrapper.LoadPemKeyFromString(signCsrReq.RcaKey);
            Org.BouncyCastle.X509.X509Certificate crt;
            if (signCsrReq.IsCa)
            {
                crt = BouncyCastleWrapper.GenerateIntermediateCertificate(csr, rca, key, 0);
            }
            else
            {
                crt = BouncyCastleWrapper.GenerateEndCertificate(csr, rca, key);
            }
            var crtString = await BouncyCastleWrapper.ToStringAsPem(crt);
            return Ok(crtString);
        }

        [HttpPost("validate")]
        public IActionResult ValidateCertificate([FromForm] CheckCrtReq checkCrtReq)
        {
            var cert = BouncyCastleWrapper.LoadPemCertFromString(checkCrtReq.Crt);
            var certChain = BouncyCastleWrapper.LoadPemCrtChaingFromString(checkCrtReq.CertChain);
            var result = BouncyCastleWrapper.ValidateCert(cert, certChain);
            return result ? Ok() : BadRequest();
        }
    }
}
